' Qsort Complete v2.3, by Agamemnus 2008-20010
' Last updated January 2010
' License: Public domain.
'
' All-purpose quicksort.
'
' Macros: 
' qsortDefine(startTypeCheck, type1)
' qsortDefineLinked(startTypeCheck, type1, type2)
' qsortDefineUDT(startTypeCheck, type1, comparesubType)
' qsortDefineLinkedUDT(startTypeCheck, type1, type2, comparesubType)
' qsortDefineBase(subNameCheck, startTypeCheck, type1, type2, comparesubType)
'
' Explanation of parameters:
' startTypeCheck - "low" or "high". A value of "low" means the array
' is sorted from lowest to highest. In any other case, it is sorted
' from highest to lowest.
' type1 -- the type (integer, ubyte, etc.) of the array to sort.
' type2 -- the type of the array linked to the array of the sorted list. (if there is
' subNameCheck -- A value of 'default' makes the qsort sub name 'qsort'.
'                 (Otherwise, the qsort sub name is
'                  whatever 'startTypeCheck' is.)
' comparesubType -- A subtype within the type to compare to.
'
' qsortDefineBase is the primary qsort macro. The other 3 macros call it
' with predefined parameters as can be seen in the macro definitions below.
'
' Functions:
' For linked/paired values:
' qsort([pointer to array 1], [pointer to array 2], count)
'
' For linked/paired values, low to high or high to low:
' qsort([pointer to array 1], [pointer to array 2], count, [low to high=0, high to low=1])
'
' For sorting a single array:
' qsort([pointer to array 1], count)
'
' For sorting a single array, low to high or high to low:
' qsort([pointer to array 1], count, [low to high=0, high to low=1])
'

#ifdef qsort_nullobj
 #undef qsort_nullobj
#endif

#macro qsortDefineUDT(qsort_startTypeCheck, qsort_type1, qsort_comparesubType)
 qsortDefineBase(qsort_default, qsort_startTypeCheck, qsort_type1, qsort_nullobj, qsort_comparesubType)
#endmacro

#macro qsortDefineLinkedUDT(qsort_startTypeCheck, qsort_type1, qsort_type2, qsort_comparesubType)
 qsortDefineBase(qsort_default, qsort_startTypeCheck, qsort_type1, qsort_type2, qsort_comparesubType)
#endmacro

#macro qsortDefine(qsort_startTypeCheck, qsort_type1)
 qsortDefineBase(qsort_default, qsort_startTypeCheck, qsort_type1, qsort_nullobj, qsort_nullobj)
#endmacro

#macro qsortDefineLinked(qsort_startTypeCheck, qsort_type1, qsort_type2)
 qsortDefineBase(qsort_default, qsort_startTypeCheck, qsort_type1, qsort_type2, qsort_nullobj)
#endmacro

#macro qsortDefineBase(qsort_subNameCheck, qsort_startTypeCheck, qsort_type1, qsort_type2, qsort_comparesubType)
#ifdef qsort_subName
#undef qsort_subName
#endif
#ifdef qsort_startType
#undef qsort_startType
#endif
#ifdef qsort_baseparams
#undef qsort_baseparams
#endif

#if (qsort_startTypeCheck=low)
 #define qsort_startType low
#else
 #define qsort_startType high
#endif


#if (qsort_subNameCheck=qsort_default)
' #ifdef ##qsort
' #undef ##qsort
' #endif
 #define qsort_subName qsort
#endif

#if (qsort_type2 <> qsort_nullobj)
 #define qsort_baseparams byval ptr1 as qsort_type1 ptr, byval ptr2 as qsort_type2 ptr, byval qsort_sortmax as integer
#else
 #define qsort_baseparams byval ptr1 as qsort_type1 ptr, byval qsort_sortmax as integer
#endif

declare sub qsort_subName overload (qsort_baseparams)
declare sub qsort_subName overload (qsort_baseparams, byval qsort_lowhigh as integer)
sub qsort_subName overload (qsort_baseparams)
qsortInnerBase(qsort_startType, qsort_type1, qsort_type2, qsort_comparesubType)
end sub

sub qsort_subName overload (qsort_baseparams, byval qsort_lowhigh as integer)
if qsort_lowhigh = 0 then
qsortInnerBase(low, qsort_type1, qsort_type2, qsort_comparesubType)
else
qsortInnerBase(high, qsort_type1, qsort_type2, qsort_comparesubType)
end if
end sub
#endmacro

#macro qsortInnerBase(qsort_startType, qsort_type1, qsort_type2, qsort_comparesubType)
qsort_sortmax-=1
dim as integer g2(0 To qsort_sortmax), h2(0 To qsort_sortmax)
dim as integer i, j, r, e, g, h
dim as qsort_type1 k
#if (qsort_type2 <> qsort_nullobj)
dim as qsort_type2 m
#endif
e = 0: g2(0) = 0: h2(0) = qsort_sortmax
qsort_startType##1: g = g2(e): h = h2(e)
qsort_startType##2: i = g: j = h: r = (g + h) \ 2
k = ptr1[r]
#if (qsort_startType=high)
 #if (qsort_comparesubType=qsort_nullobj)
qsort_startType##3: if ptr1[i] > k then i += 1: goto qsort_startType##3
qsort_startType##4: if ptr1[j] < k then j -= 1: goto qsort_startType##4
 #else
qsort_startType##3: if ptr1[i].##qsort_comparesubType > k.##qsort_comparesubType then i += 1: goto qsort_startType##3
qsort_startType##4: if ptr1[j].##qsort_comparesubType < k.##qsort_comparesubType then j -= 1: goto qsort_startType##4
 #endif
#else
 #if (qsort_comparesubType=qsort_nullobj)
qsort_startType##3: if ptr1[i] < k then i += 1: goto qsort_startType##3
qsort_startType##4: if ptr1[j] > k then j -= 1: goto qsort_startType##4
 #else
qsort_startType##3: if ptr1[i].##qsort_comparesubType < k.##qsort_comparesubType then i += 1: goto qsort_startType##3
qsort_startType##4: if ptr1[j].##qsort_comparesubType > k.##qsort_comparesubType then j -= 1: goto qsort_startType##4
 #endif
#endif
if i <= j then
swap ptr1[i], ptr1[j]
#if (qsort_type2 <> qsort_nullobj)
swap ptr2[i], ptr2[j]
#endif
i += 1: j -= 1
if i <= j then goto qsort_startType##3
end if
if j - g + i < h then
if i < h then g2(e) = i: h2(e) = h: e += 1
h = j
else
if g < j then g2(e) = g: h2(e) = j: e += 1
g = i
end if
if g < h then goto qsort_startType##2 else e -= 1: if e <> -1 then goto qsort_startType##1
erase g2, h2
#endmacro



function getId overload (byref stringToSearch as string ptr, byref stringToSearchFor as string, byval minval as integer, byval maxval as integer) as integer
 dim as integer n
 do
  if minval > maxval then exit do
  n = (minval+maxval) / 2
  if stringToSearchFor = stringToSearch[n] then return n
  if stringToSearchFor < stringToSearch[n] then
   maxval = n - 1
  else
   minval = n + 1
  end if
 loop
 return -1
end function

#macro iniGetId (udt, comparisonSubtype)
function getId overload (byref customudt as udt ptr, byref stringToSearchFor as string, byval minval as integer, byval maxval as integer) as integer
 dim as integer n
 do
  if minval > maxval then exit do
  n = (minval+maxval) / 2
  if stringToSearchFor = customudt[n].comparisonSubType then return n
  if stringToSearchFor < customudt[n].comparisonSubType then
   maxval = n - 1
  else
   minval = n + 1
  end if
 loop
 return -1
end function
#endmacro

'qsortDefineLinked(low, integer, string)
'cls
'randomize timer
'dim as double t1, t2
'dim as integer i, j, sortmin, sortmax, temp
'sortmin = -10
'sortmax = 10
'dim as integer array1(sortmin to sortmax)
'dim as string array2(sortmin to sortmax)

'for i = sortmin to sortmax
'temp = int(rnd * 10) + 1
'array2(i) = space(temp)
'for j = 1 to temp
'if int(rnd * 2) then
'mid(array2(i), j, 1) = chr(int(rnd * 26) + 97)
'else
'mid(array2(i), j, 1) = chr(int(rnd * 26) + 65)
'end if
'next j
'array1(i) = (int(rnd * 10000) - 5000)
'print array1(i);" "; array2(i)
'next i
't1 = timer
'qsort @array1(sortmin), @array2(sortmin), sortmax-sortmin+1, 0
't2 = timer
'print
'for i = sortmin to sortmax
'print array1(i);" "; array2(i)
'next i
'print: print t2-t1
'sleep
'system
